"""Compare similarity scores which one set of GloVe embeddings generates
versus another set of GloVe embeddings to determine whether the two sets
of embeddings are more or less the same."""

import pandas as pd
from gensim.models import KeyedVectors
from scipy.stats import spearmanr
from scipy.spatial.distance import cosine

stanford_path = "/workspace/students/reichelt/ba_data/embeddings/glove/glove.6B.300d_w2vformat.txt"
dd_glove_path = "/workspace/students/reichelt/ba_data/embeddings/glove/dd-glove/english_vectors_no_debiasing.txt"

stanford_model = KeyedVectors.load_word2vec_format(stanford_path, binary=False)
stanford_model.init_sims(replace=True)
dd_glove_model = KeyedVectors.load_word2vec_format(dd_glove_path, binary=False)
stanford_model.init_sims(replace=True)

ws353_data = pd.read_csv("/home/students/reichelt/ba/bias-mitigation-ba/WordSim353/data/original_finkelstein/combined.csv")

def calculate_embedding_similarity(row, embedding):
    if embedding=="stanford":
        model = stanford_model
    elif embedding=="dd":
        model = dd_glove_model
    else:
        raise ValueError

    word_1 = row["Word 1"].lower()
    word_2 = row["Word 2"].lower()

    if word_1 in model:
        embd_1 = model[word_1]
    else:
        print(f"{word_1} not in vocab. returning similarity 0.")
        return 0.0
    if word_2 in model:
        embd_2 = model[word_2]
    else:
        print(f"{word_2} not in vocab. returning similarity 0.")
        return 0.0

    return 1 - cosine(embd_1, embd_2)

def calculate_stanford(row):
    word_1 = row["Word 1"].lower()
    word_2 = row["Word 2"].lower()
    if word_1 in stanford_model:
        embd_1 = stanford_model[word_1]
    else:
        print(f"{word_1} not in vocab. returning similarity 0.")
        return 0.0
    if word_2 in stanford_model:
        embd_2 = stanford_model[word_2]
    else:
        print(f"{word_2} not in vocab. returning similarity 0.")
        return 0.0
    return 1 - cosine(embd_1, embd_2)

def calculate_dd_glove(row):
    word_1 = row["Word 1"].lower()
    word_2 = row["Word 2"].lower()
    if word_1 in dd_glove_model:
        embd_1 = dd_glove_model[word_1]
    else:
        print(f"{word_1} not in vocab. returning similarity 0.")
        return 0.0
    if word_2 in dd_glove_model:
        embd_2 = dd_glove_model[word_2]
    else:
        print(f"{word_2} not in vocab. returning similarity 0.")
        return 0.0
    return 1 - cosine(embd_1, embd_2)

ws353_data["Stanford Score"] = ws353_data.apply(calculate_stanford, axis=1)
ws353_data["DD-GloVe Score"] = ws353_data.apply(calculate_dd_glove, axis=1)

rho, p = spearmanr(
        ws353_data["Stanford Score"], ws353_data["DD-GloVe Score"],
        nan_policy="raise"
    )

print("rho: " + str(rho))
print("p: " + str(p))
