"""Check how often the most common names of each nationality
occur in the Wikipedia snapshot used for GloVe training.
To be used when there is no vocab count file yet."""

import pandas as pd

df = pd.read_csv("../data/names_nationality.csv", usecols=["name", "nationality", "gender"])
df["occurrences_in_wikipedia"] = 0

CHUNK_SIZE = 1024 * 1024  # 1 MB

with open("../data/wikipedia/wikipedia_corpus.txt", "rb") as f:
    CHUNK_NO = 1
    while True:
        print(f"reading Wikipedia data: approx. {CHUNK_NO} MB/8100 MB", end="\r")
        wikipedia_text = f.read(CHUNK_SIZE)
        if not wikipedia_text:
            print("reading completed")
            break

        for index, row in df.iterrows():
            name = row['name'].encode()
            count = wikipedia_text.count(name)
            df.at[index, "occurrences_in_wikipedia"] += count
        
        CHUNK_NO += 1

df.to_csv("../data/names_nationality_wikipedia.csv", index=False)

avg_occurrences = df.groupby("nationality")["occurrences_in_wikipedia"].mean()
for nationality, avg_count in avg_occurrences.items():
    print(f"Nationality: {nationality}, Average Occurrences in Wikipedia: {avg_count:.2f}")
