"""Get origin specific words by looking up
origin-definitional words in dictionary entries."""

import json
import typing
import pandas as pd


def check_definitions(word_list: typing.List[str]) -> set:
    dduw = pd.read_csv("/home/students/reichelt/ba/bias-mitigation-ba/data/dduw.csv", sep=";", encoding="utf-8")
    pattern = ' | '.join(word_list)  # regex pattern matching any of the definitional words

    # str.contains() filters any rows where >= 1 definitional word is present
    relevant_entries = dduw[dduw["Definition/Erklärung"].notna() &
                            dduw["Term-Label"].notna() &
                            dduw["Definition/Erklärung"].str.contains(
                            pattern, case=True, regex=True)]

    # Term-Label == 1 ensures only the first senses/definitions for each lemma are considered
    # not sure why it doesn't work when including in first query, but it is what it is
    relevant_entries = relevant_entries[relevant_entries["Term-Label"] == "1"]

    # to make sure each word occurs only once
    lemma_values = list(set(relevant_entries['Lemma'].tolist()))
    return lemma_values

if __name__ == "__main__":
    definitional_words = ["polnisch", "polnisches", "polnische", "polnischer",
                          "polnischsprachig", "polnischstämmig", "Pole", "Polen",
                          "Polin", "Polens", "Polinnen",
                          "deutsch", "deutsches", "deutsche", "deutschen",
                          "deutschsprachig", "deutschstämmig", "Deutschlands",
                          "Deutschland", "Deutscher", "Deutsche", "Deutschen"]

    origin_specific_words = check_definitions(definitional_words)
    lowercased_words = [w.lower() for w in origin_specific_words]

    with open("/home/students/reichelt/ba/bias-mitigation-ba/debiaswe-master/data/polish/bias_specific_seed.json", mode="w", encoding="utf-8") as f:
        json.dump(lowercased_words, f)
